<?php
namespace BancoTalentos\Services;

use BancoTalentos\Util\GoogleAuth;
use BancoTalentos\Repositories\OperadorRepository;
use BancoTalentos\Services\OperadorService;;
use BancoTalentos\Util\RequestUtils;
use BancoTalentos\Util\Util;
use GuzzleHttp\Client;
use Doctrine\ORM\EntityManager;

class LoginService {
    private const PASSWORD_ENDING = 'e86a8d5528374ad6470ca9f244dcfa22';

    /** @var GoogleAuth */
    protected $googleAuth;

    /** @var OperadorService */
    protected $operadorService;

    /** @var OperadorRepository */
    protected $operadorRepository;

    /** @var string */
    protected $productId;

    /** @var string */
    protected $loginUrl;

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        GoogleAuth $googleAuth,
        OperadorService $operadorService,
        OperadorRepository $operadorRepository,
        string $productId,
        string $projectPath,
        string $loginPath,
        EntityManager $entityManager
    ) {
        $this->googleAuth = $googleAuth;
        $this->operadorService = $operadorService;
        $this->operadorRepository = $operadorRepository;
        $this->productId = $productId;
        $this->loginUrl = $projectPath . $loginPath;
        $this->entityManager = $entityManager;
    }

    public function loginGoogle(
        string $idToken,
        string $requesterUrl
    ) {
        $loginResponse = null;

        $tokenData = $this->googleAuth->fetchDataFromIdToken($idToken);
        $this->googleAuth->verifyTokenData($tokenData);

        $googleId = (string) $tokenData['sub'];

        $operador = $this->getOperador($tokenData);
        $createdOper = $operador['__is_new'];

        $nrOrg          = (int)    $operador['NRORG'];
        $cdOperador     = (string) $operador['CDOPERADOR'];
        $email          = (string) $operador['DSEMAILOPER'];
        $oldPassword    = (string) $operador['CDSENHAOPERWEB'];

        $password = $this->getGooglePassword($googleId);

        if (!$createdOper) {
            $passwordEncrypt = md5($password);
            $this->changePassword($nrOrg, $cdOperador, $passwordEncrypt);
        }

        try {
            $loginResponse = $this->login($email, $password, $requesterUrl);
        } finally {
            if (!$createdOper) {
                $this->changePassword($nrOrg, $cdOperador, $oldPassword);
            }
        }


        return $loginResponse;
    }

    private function getOperador(
        array $tokenData
    ): array {
        $googleId       = (string) $tokenData['sub'];
        $googleEmail    = (string) $tokenData['email'];
        $googleName     = (string) $tokenData['name'];

        $paramOperExists = $this->verifyParamOper($googleId);
        $operador = null;
        $createdOper = false;

        if ($paramOperExists) {
            $operador = $this->operadorRepository->getOperadorByGoogleId($googleId);
        } else {
            $operador = $this->operadorRepository->getOperadorByEmail($googleEmail);
        }

        $this->entityManager->beginTransaction();

        try {
            if (!$operador) {
                $operador = $this->createOperadorWithGoogleData($googleId, $googleEmail, $googleName);
                $createdOper = true;
            }

            $nrOrg = $operador['NRORG'];
            $cdOperador = $operador['CDOPERADOR'];

            if (!$paramOperExists) {
                $this->createParamOper($nrOrg, $cdOperador, $googleId);
            }

            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }

        $operador['__is_new'] = $createdOper;

        return $operador;
    }

    private function getGooglePassword(
        string $googleId
    ): string {
        return ($googleId . self::PASSWORD_ENDING);
    }

    private function createOperadorWithGoogleData(
        string $googleId,
        string $googleEmail,
        string $googleName
    ): array {
        $password = $this->getGooglePassword($googleId);

        $row = [
            'NMOPERADOR' => $googleName,
            'DSEMAILOPER' => $googleEmail,
            'PASSWORD' => $password,
            'PASSWORD_CONFIRMATION' => $password,
        ];

        $cdOperador = $this->operadorService->setOperadorAcesso($row);
        $operador = $this->operadorRepository->get(Util::DEFAULT_ORG, $cdOperador);

        return $operador;
    }

    public function changePassword(
        int $nrOrg,
        string $cdOperador,
        string $password
    ): void {
        $params = [
            'P_NRORG' => $nrOrg,
            'P_CDOPERADOR' => $cdOperador,
            'P_SENHA' => $password,
        ];

        $this->operadorRepository->changePassword($params);
    }

    public function login(
        string $email,
        string $password,
        string $requesterUrl
    ): array {
        $route = $this->loginUrl . '/login';

        $params = [
            'requestType' => 'FilterData',
            'filter' => [
                [
                    'name' => 'EMAIL',
                    'operator' => '=',
                    'value' => $email,
                ],
                [
                    'name' => 'PASSWORD',
                    'operator' => '=',
                    'value' => $password,
                ],
                [
                    'name' => 'PRODUCT_ID',
                    'operator' => '=',
                    'value' => $this->productId,
                ],
                [
                    'name' => 'REQUESTER_URL',
                    'operator' => '=',
                    'value' => $requesterUrl,
                ],
                [
                    'name' => 'ATTEMPTS',
                    'operator' => '=',
                    'value' => 1,
                ],
                [
                    'name' => 'KEEP_CONNECTED',
                    'operator' => '=',
                    'value' => 'S',
                ],
            ],
        ];

        $http = new Client([
            'base_uri' => $route
        ]);

        $response = $http->request('POST', $route, [
            'json' => $params,
            'headers' => [
                'Content-Type: application/json',
                'Content-Length: ' . strlen(json_encode($params)),
            ],
        ]);

        $content = $response->getBody()->getContents();
        $responseData = json_decode($content, true);

        if ($responseData['dataset']['userData'] ?? false) {
            $cookiePHP = $response->getHeaders()['Set-Cookie'][0];
            header('Set-Cookie: ' . $cookiePHP);
        }

        return $responseData;
    }

    private function verifyParamOper(
        string $googleId
    ) {
        $paramOper = $this->operadorRepository->getParamOperByGoogleId($googleId);
        $paramOperExists = $paramOper !== false;

        return $paramOperExists;
    }

    private function createParamOper(
        int $nrOrg,
        string $cdOperador,
        string $googleId
    ): void {
        $params = [
            'NRORG' => $nrOrg,
            'CDOPERADOR' => $cdOperador,
            'DSGOOGLEID' => $googleId,
        ];

        $this->operadorRepository->createParamOper($params);
    }

}